package com.chinaitop.depot.grainAPI.controller;

import com.chinaitop.depot.common.JsonToObjectUtils;
import com.chinaitop.depot.common.ResponseEntity;
import com.chinaitop.depot.grainAPI.entity.TTestdataCloud;
import com.chinaitop.depot.grainAPI.service.ITTestdataCloudService;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 文件描述
 *
 * @author tongxi.xia
 * @date 2019年12月26日 14:48
 */
@RestController
@RequestMapping(value = "/depotGrain")
public class GrainAPIController {

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private ITTestdataCloudService testdataCloudService;

    final static Logger logger = LoggerFactory.getLogger(GrainAPIController.class);

    /**
     * @MethodName: sendTemperatureMeasurementInstruction
     * @Description: TODO 市平台测温接口，根据货位编码检测测温
     * @Param: tType 检测类型(检测类型0测温，1粮油,2测水分)
     * @Param: hwCode 仓房编码
     * @Return: java.lang.String
     * @Author: tongxi.xia
     * @Date: 2019/12/26
     **/
    @GetMapping("/sendTemperatureMeasurementInstruction")
    public ResponseEntity sendTemperatureMeasurementInstruction(@RequestParam String hwCode, @RequestParam String tType) {
        String returnStr = "";
        try {
            List<Map<String, Object>> depotUrl = testdataCloudService.getDepotUrl(hwCode);
            if (depotUrl != null && depotUrl.size() > 0) {
                Map<String, Object> objectMap = depotUrl.get(0);
                String depoturl = objectMap.get("depoturl").toString();
                String cfCode = objectMap.get("cfcode").toString();
                String url = depoturl + "/intelligents/temperatureRecord/sendOnlyStoreDetection";
//                String url = "http://localhost:8765/depot-intelligent/intelligents/temperatureRecord/sendOnlyStoreDetection";
                MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
                requestEntity.add("storeCode", cfCode);
                requestEntity.add("tType", tType);
                String s = restTemplate.postForObject(url, requestEntity, String.class);
                ObjectMapper mapper = new ObjectMapper();
                mapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
                ResponseEntity responseEntity = mapper.readValue(s, ResponseEntity.class);
                returnStr = responseEntity.getData().toString();
            }
        } catch (Exception e) {
            logger.info("市平台测温接口sendTemperatureMeasurementInstruction检测失败：参数：hwCode=" + hwCode + " :tType:" + tType);
            return ResponseEntity.failed("检测失败！");
        }
        logger.info("市平台测温接口sendTemperatureMeasurementInstruction调用成功：参数：hwCode=" + hwCode + " :tType:" + tType);
        return ResponseEntity.ok(returnStr);
    }

    /**
     * @MethodName: 接受终端测温数据
     * @Description: TODO
     * @Param:
     * @Return:
     * @Author: tongxi.xia
     * @Date: 2020/1/11
     **/
    @PostMapping("/acceptTemperatureData")
    public void acceptTemperatureData(@RequestBody String grainJson) {
        try {
            if (StringUtils.isNotBlank(grainJson)) {
                ArrayList<TTestdataCloud> tTestdataClouds = new ArrayList<>();
                System.out.println(grainJson);
                ArrayList<Object> objects = JsonToObjectUtils.jsonToObjectsDiverse(grainJson, TTestdataCloud.class, "yyyy-MM-dd HH:mm:ss");
                objects.forEach(obj -> {
                    TTestdataCloud s = (TTestdataCloud) obj;
                    tTestdataClouds.add(s);
                });
                testdataCloudService.saveBatch(tTestdataClouds);
            }
            logger.info("接受终端测温数据acceptTemperatureData保存成功：参数：grainJson=" + grainJson);
        } catch (Exception e) {
            logger.info("接受终端测温数据acceptTemperatureData保存失败：参数：grainJson=" + grainJson);
            e.printStackTrace();
        }
    }

    @GetMapping("/test")
    public ResponseEntity test() {
        return ResponseEntity.ok("api测试成功");
    }
}
