package com.chinaitop.depot.utils;

import com.chinaitop.depot.common.ResponseEntity;
import com.chinaitop.depot.common.TypeEnum;
import org.springframework.stereotype.Component;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.Iterator;
import java.util.Map;

/**
 * @author qingsong.han
 * @description: 云端远程检测控制工具类
 * @create 2020-06-01 11:05
 */
@Component
public class RemoteControlUtil {
    /**
     * rest接口注入
     */
    @Resource
    private RestTemplate restTemplate;
    /**
     * url常量
     */
    // 数量检测,获取测量进度
    public static final String scanProgress = "/intelligents/monitorWarning/getScanResult";
    // 数量检测,获取扫描结果
    public static final String measureWeight = "/intelligents/monitorWarning/getmeasureweight";
    // 数量检测,监测指令接口
    public static final String sendOrder = "/intelligents/monitorWarning/sendNumberMonitor";
    // 粮温检测-手动选仓
    public static final String chooseCFCode = "/intelligents/temperatureRecord/sendTemperatureDetection";
    // 粮情全仓检测
    public static final String foodAll = "/intelligents/temperatureRecord/sendTemperatureDetectionAll";
    // 粮情单仓检测
    public static final String foodOnly = "/intelligents/temperatureRecord/sendOnlyStoreDetection";
    // 气体全仓检测
    public static final String gasAll = "/intelligents/gasDetection/sendGasDetectionAll";
    // 气体单仓检测
    public static final String gasOnly = "/intelligents/gasDetection/sendOnlyGasDetection";
    // 气象检测
    public static final String weatherMonitored = "/intelligents/wertherinfo/sendNowWeatherDetection";
    // 液位所有油罐检测
    public static final String levelAll = "/intelligents/LevelDetection/sendLevelDetectionAll";
    // 液位单油罐检测
    public static final String onlyLevel = "/intelligents/LevelDetection/sendOnlyStoreDetection";
    // 所有虫害检测
    public static final String pestAll = "/intelligents/insectPestDetection/sendInsectPestDetectionAll";
    // 单仓虫害检测
    public static final String onlyPest = "/intelligents/insectPestDetection/sendOnlyInsectPestDetection";
    // 能耗检测
    public static final String energyOrder = "/intelligents/energyDetection/addHistoryData";

    /**
     * 远程库端请求方法
     *
     * @param url           库端地址
     * @param requestEntity post请求参数 无参数传入 new LinkedMultiValueMap<>()
     * @param httpType      http 类型
     * @param requestMap    get请求参数 无参数传入 new HashMap<>()
     * @param <K>           key
     * @param <V>           value
     * @return string
     */
    public <K, V> String remoteScan(@NotNull String url, MultiValueMap<K, V> requestEntity, @NotNull String httpType, Map<K, V> requestMap) {
        httpType = httpType.toUpperCase();
        // 初始设备通讯异常
        String result = TypeEnum.S_STX.getDesc();
        ResponseEntity responseEntity;
        try {
            if ("GET".equals(httpType)) {
                responseEntity = restTemplate.getForObject(getForObject(url, requestMap), ResponseEntity.class);
            } else {
                responseEntity = restTemplate.postForObject(url, requestEntity, ResponseEntity.class);
            }
            if (!StringUtils.isEmpty(responseEntity)) {
                result = responseEntity.getData().toString();
            }
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return result;
        }
    }

    /**
     * 封装的get请求，暂时只支持map传参，并且value只支持基本类型和String
     *
     * @param url    库端配置url
     * @param object 接收数据(只支持map类型)
     * @return string类型带参数get请求url
     */
    private static String getForObject(String url, Object object) {
        StringBuilder stringBuilder = new StringBuilder(url);
        if (object instanceof Map) {
            Iterator iterator = ((Map) object).entrySet().iterator();
            if (iterator.hasNext()) {
                stringBuilder.append("?");
                Object element;
                while (iterator.hasNext()) {
                    element = iterator.next();
                    Map.Entry entry = (Map.Entry) element;
                    //过滤value为null，value为null时进行拼接字符串会变成 "null"字符串
                    if (entry.getValue() != null) {
                        stringBuilder.append(element).append("&");
                    }
                    url = stringBuilder.substring(0, stringBuilder.length() - 1);
                }
            }
        } else {
            throw new RuntimeException("url请求:" + url + "请求参数有误不是map类型");
        }
        return url;
    }

}
